---
--- Run as defined rw user on newly created database
--- psql -U cdnskey_processor -d cdnskey_processor -f 0007_get_raw_scan_results.sql
---

CREATE FUNCTION get_raw_scan_results (requested_uid TEXT, requested_fqdn TEXT, acceptance_period INTERVAL)
        RETURNS TABLE (
               worker_name TEXT,
               nameserver TEXT,
               nameserver_ip INET,
               scan_at TIMESTAMPTZ,
               cdnskey_status CDNSKEY_STATUS,
               cdnskey_flags INTEGER,
               cdnskey_proto INTEGER,
               cdnskey_alg INTEGER,
               cdnskey_public_key TEXT
        )
AS $$

WITH
latest_domain_jobs AS NOT MATERIALIZED (
      SELECT j.id AS scan_job_id, scan_type, nameserver,
             domain_uid, FIRST_VALUE(domain_uid) OVER w AS most_recent_domain_uid,
             scan_queue_id
        FROM scan_job AS j
             JOIN scan_queue AS q
                  ON j.scan_queue_id = q.id
       WHERE q.start_at > (CURRENT_TIMESTAMP - acceptance_period)
             AND (domain_fqdn = requested_fqdn
                  OR domain_uid = requested_uid)
      WINDOW w AS (ORDER BY start_at DESC)
)
, relevant_results AS NOT MATERIALIZED (
      SELECT scan_type, w.name AS worker_name, nameserver, nameserver_ip, scan_at,
             cdnskey_status, cdnskey_flags, cdnskey_proto, cdnskey_alg, cdnskey_public_key,
             scan_queue_id, FIRST_VALUE(scan_queue_id) OVER w AS most_recent_scan_queue_id
        FROM scan_result AS r
             INNER JOIN latest_domain_jobs AS j
                   ON j.scan_job_id = r.scan_job_id
             INNER JOIN worker AS w
                   ON w.id = r.worker_id
       WHERE domain_uid = most_recent_domain_uid
      WINDOW w AS (ORDER BY scan_at DESC)
       ORDER BY worker_id, scan_at DESC
)
SELECT worker_name, nameserver, nameserver_ip, scan_at,
       cdnskey_status, cdnskey_flags, cdnskey_proto, cdnskey_alg, cdnskey_public_key
  FROM relevant_results
 WHERE CASE WHEN scan_type = 'secure'::SCAN_TYPE
            THEN scan_queue_id = most_recent_scan_queue_id
            ELSE TRUE
       END

$$ LANGUAGE SQL;
