/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/cfg/database.hh"
#include "src/cfg/validator.hh"

#include "config.h"

#include <vector>

using namespace CdnskeyProcessor::Cfg;

boost::program_options::options_description CdnskeyProcessor::Cfg::get_options(Database& cfg)
{
    namespace po = boost::program_options;
    po::options_description options("Database options", program_options_line_length);
    options.add_options()
        ("database.host",
         po::value<std::string>(&cfg.host), "Database host")
        ("database.host_addr",
         po::value<boost::asio::ip::address>()->notifier(
            [&cfg](const auto& address)
            {
                cfg.host_addr = address;
            }),
         "Database host address (IPv4 or IPv6)")
        ("database.port",
         po::value<std::uint16_t>()->notifier(
            [&cfg](const auto& port)
            {
                cfg.port = port;
            }),
         "Database port")
        ("database.db_name",
         po::value<std::string>(&cfg.db_name), "Name of a database")
        ("database.user",
         po::value<std::string>(&cfg.user), "User of a database")
        ("database.password",
         po::value<std::string>(&cfg.password), "User's password for the database")
        ("database.connect_timeout",
         po::value<std::chrono::seconds>()->notifier(
            [&cfg](const auto& timeout)
            {
                cfg.connect_timeout = timeout;
            }),
         "Database connect timeout in seconds");
    return options;
}
