/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef LOG_HH_7C8AF3EBA27F02F485333CD89B88C396//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define LOG_HH_7C8AF3EBA27F02F485333CD89B88C396

#include <boost/optional.hpp>
#include <boost/program_options.hpp>

#include <string>

namespace CdnskeyProcessor {
namespace Cfg {

struct Log
{
    enum class Level
    {
        trace,
        debug,
        info,
        warning,
        error,
        critical
    };
    Level min_severity;

    struct Syslog
    {
        bool enabled = false;
        boost::optional<int> facility;
        boost::optional<Level> min_severity;
    };
    Syslog syslog;

    struct Console
    {
        bool enabled = false;
        boost::optional<Level> min_severity;
    };
    Console console;

    struct File
    {
        bool enabled = false;
        std::string file_name;
        boost::optional<Level> min_severity;
    };
    File file;
};

boost::program_options::options_description get_options(Log&);

void set_up_logging(const Log&);

}//namespace CdnskeyProcessor::Cfg
}//namespace CdnskeyProcessor

#endif//LOG_HH_7C8AF3EBA27F02F485333CD89B88C396
