/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef WORKER_HH_856BAA17380676E62A0BC5D24A8CE580//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define WORKER_HH_856BAA17380676E62A0BC5D24A8CE580

#include "src/cfg/log.hh"

#include <boost/optional.hpp>

#include <cstdint>
#include <string>
#include <vector>

namespace CdnskeyProcessor {
namespace Cfg {

struct Worker
{
    struct Help {};
    std::string name;
    std::string scanner;
    std::vector<std::string> scanner_args;
    struct Amqp
    {
        std::uint16_t prefetch_count;
        struct Broker
        {
            std::string hostname;
            std::uint16_t port;
            std::string vhost;
            std::string username;
            std::string password;
        } broker;
        struct Exchange
        {
            std::string insecure_fanout;
            std::string response;
        } exchange;
        struct Queue
        {
            std::string results;
            std::string insecure_prefix;
            std::string secure;
        } queue;
    } amqp;
};

Worker get_cfg(int argc, char** argv, void(*)(const Log&) = nullptr);

}//namespace CdnskeyProcessor::Cfg
}//namespace CdnskeyProcessor

#endif//WORKER_HH_856BAA17380676E62A0BC5D24A8CE580
