/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef COMMON_TYPES_HH_06A7EEBFE6376203BAC959B4AF5F6CE7//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define COMMON_TYPES_HH_06A7EEBFE6376203BAC959B4AF5F6CE7

#include <libstrong/type.hh>

#include <boost/asio/ip/address.hpp>

#include <chrono>
#include <cstdint>
#include <string>

namespace CdnskeyProcessor {

template <typename Tag>
using StrongId = LibStrong::TypeWithSkills<
        std::uint64_t,
        Tag,
        LibStrong::Skill::Comparable,
        LibStrong::Skill::Streamable>;

template <typename Tag>
using StrongSignedId = LibStrong::TypeWithSkills<
        std::int64_t,
        Tag,
        LibStrong::Skill::Comparable,
        LibStrong::Skill::Streamable>;

template <typename Tag>
using StrongString = LibStrong::TypeWithSkills<
        std::string,
        Tag,
        LibStrong::Skill::Comparable,
        LibStrong::Skill::Streamable>;

using ScanJobId = StrongSignedId<struct ScanJobIdTag_>;
using ScanQueueId = StrongId<struct ScanQueueIdTag_>;
using ScanBatchId = StrongSignedId<struct ScanBatchIdTag_>;
using DomainFqdn = StrongString<struct DomainFqdnTag_>;
using DomainUid = StrongString<struct DomainUidTag_>;
using Nameserver = StrongString<struct NameserverTag_>;
using WorkerName = StrongString<struct WorkerNameTag_>;
using Timestamp = LibStrong::Type<std::chrono::system_clock::time_point, struct TimestampTag_>;
using IpAddress = LibStrong::Type<boost::asio::ip::address, struct IpAddressTag_>;
using GrpcAddress = StrongString<struct GrpcAddressTag_>;
using MaxScanPeriod = LibStrong::Type<std::chrono::seconds, struct MaxScanPeriodTag_>;
using MinPeriodFixedCdnskey = LibStrong::Type<std::chrono::seconds, struct MinPeriodFixedCdnskeyTag_>;

enum class ScanType
{
    insecure,
    secure
};

}//namespace CdnskeyProcessor

#endif//COMMON_TYPES_HH_06A7EEBFE6376203BAC959B4AF5F6CE7
