/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/internal/amqp/boost_asio_handler.hh"

using namespace CdnskeyProcessor::Internal::Amqp;

std::unique_ptr<AMQP::TcpHandler> CdnskeyProcessor::Internal::Amqp::make_boost_asio_handler(
        boost::asio::io_context& service,
        std::function<void(AMQP::TcpConnection*)> on_ready,
        std::function<void(AMQP::TcpConnection*, const char*)> on_error,
        std::function<void(AMQP::TcpConnection*)> on_lost)
{
    class IoHandler : public AMQP::LibBoostAsioHandler
    {
    public:
        using AMQP::LibBoostAsioHandler::LibBoostAsioHandler;
        IoHandler& set_on_ready(std::function<void(AMQP::TcpConnection*)> fnc)
        {
            on_ready_ = fnc;
            return *this;
        }
        IoHandler& set_on_error(std::function<void(AMQP::TcpConnection*, const char*)> fnc)
        {
            on_error_ = fnc;
            return *this;
        }
        IoHandler& set_on_lost(std::function<void(AMQP::TcpConnection*)> fnc)
        {
            on_lost_ = fnc;
            return *this;
        }
    private:
        void onReady(AMQP::TcpConnection* connection) override
        {
            if (on_ready_)
            {
                on_ready_(connection);
            }
        }
        void onError(AMQP::TcpConnection* connection, const char *message) override
        {
            if (on_error_)
            {
                on_error_(connection, message);
            }
        }
        void onLost(AMQP::TcpConnection* connection) override
        {
            if (on_lost_)
            {
                on_lost_(connection);
            }
        }
        uint16_t onNegotiate(AMQP::TcpConnection*, uint16_t) override
        {
            return 0;//heartbeat turned off
        }
        std::function<void(AMQP::TcpConnection*)> on_ready_ = nullptr;
        std::function<void(AMQP::TcpConnection*, const char*)> on_error_ = nullptr;
        std::function<void(AMQP::TcpConnection*)> on_lost_ = nullptr;
    };
    auto io_handler = std::make_unique<IoHandler>(service);
    io_handler->set_on_ready(on_ready)
               .set_on_error(on_error)
               .set_on_lost(on_lost);
    return io_handler;
}
