/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef EXCHANGE_HH_ED3D534E2BF5E802A154EA64FDE6F4A6
#define EXCHANGE_HH_ED3D534E2BF5E802A154EA64FDE6F4A6

#include <libstrong/type.hh>

#include <string>
#include <tuple>
#include <utility>

namespace CdnskeyProcessor {
namespace Internal {
namespace Amqp {

struct Recipient
{
    std::string exchange_name;
    std::string routing_key;
};

template <typename Tag = void>
class RoutingKey
{
public:
    explicit RoutingKey(std::string value)
        : value_{std::move(value)}
    { }
    operator const std::string&() const& noexcept
    {
        return value_;
    }
    operator std::string&&() && noexcept
    {
        return std::move(value_);
    }
private:
    std::string value_;
};

struct Fanout;

template <typename ...Recipients>
struct Direct;

using ExchangeName = LibStrong::Type<std::string, struct ExchangeNameTag_>;

template <typename> class Exchange;

template <>
class Exchange<Fanout>
{
public:
    explicit Exchange(ExchangeName exchange_name)
        : exchange_name_{*std::move(exchange_name)}
    { }
    operator Recipient() const;
private:
    std::string exchange_name_;
};

using FanoutExchange = Exchange<Fanout>;

template <typename ...Recipients>
class Exchange<Direct<Recipients...>>
{
public:
    explicit Exchange(ExchangeName exchange_name, Recipients ...recipients)
        : exchange_name_{*std::move(exchange_name)},
          recipients_{std::move(recipients)...}
    { }
    template <typename R>
    Recipient recipient() const
    {
        return Recipient{exchange_name_, std::get<R>(recipients_)};
    }
private:
    std::string exchange_name_;
    std::tuple<Recipients...> recipients_;
};

template <>
class Exchange<Direct<>>
{
public:
    explicit Exchange(ExchangeName exchange_name)
        : exchange_name_{*std::move(exchange_name)}
    { }
    Recipient recipient(std::string recipient) const;
private:
    std::string exchange_name_;
};

template <typename ...Recipients>
using DirectExchange = Exchange<Direct<Recipients...>>;

}//namespace CdnskeyProcessor::Internal::Amqp
}//namespace CdnskeyProcessor::Internal
}//namespace CdnskeyProcessor

#endif//EXCHANGE_HH_ED3D534E2BF5E802A154EA64FDE6F4A6
