/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef QUEUE_HH_161FBB76754B14894E0290065606762F
#define QUEUE_HH_161FBB76754B14894E0290065606762F

#include <liblog/liblog.hh>

#include <iosfwd>
#include <string>

namespace CdnskeyProcessor {
namespace Internal {
namespace Amqp {

class Queue
{
public:
    explicit Queue(std::string name);
    operator const std::string&() const noexcept;
private:
    std::string queue_name_;
    friend std::ostream& operator<<(std::ostream&, const Queue&);
};

std::ostream& operator<<(std::ostream& out, const Queue& queue);

}//namespace CdnskeyProcessor::Internal::Amqp
}//namespace CdnskeyProcessor::Internal
}//namespace CdnskeyProcessor

template <typename Char>
struct fmt::formatter<CdnskeyProcessor::Internal::Amqp::Queue, Char>
{
    template <typename ParseContext>
    constexpr auto parse(ParseContext& ctx)
    {
        return ctx.begin();
    }
    template <typename FormatContext>
    auto format(const CdnskeyProcessor::Internal::Amqp::Queue& value, FormatContext& ctx) const -> decltype(ctx.out())
    {
        return fmt::format_to(ctx.out(), "{}", static_cast<const std::string&>(value));
    }
};

#endif//QUEUE_HH_161FBB76754B14894E0290065606762F
