/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/internal/amqp/tcp_connection.hh"

using namespace CdnskeyProcessor::Internal::Amqp;

std::unique_ptr<AMQP::TcpConnection> CdnskeyProcessor::Internal::Amqp::make_tcp_connection(
        std::unique_ptr<AMQP::TcpHandler> io_handler,
        const AMQP::Address& address)
{
    using TcpHandlerStorage = std::unique_ptr<AMQP::TcpHandler>;
    class Connection : private TcpHandlerStorage, public AMQP::TcpConnection
    {
    public:
        explicit Connection(
                std::unique_ptr<AMQP::TcpHandler> io_handler,
                const AMQP::Address& address)
            : TcpHandlerStorage{std::move(io_handler)},
              AMQP::TcpConnection{this->TcpHandlerStorage::get(), address}
        { }
    };
    return std::make_unique<Connection>(std::move(io_handler), address);
}
