/*
 * Copyright (C) 2020-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/internal/scan_result.hh"
#include "src/unwrapper/unwrapper.hh"
#include "src/wrapper/wrapper.hh"

namespace CdnskeyProcessor {

bool ScanResult::is_successful() const
{
    return CdnskeyProcessor::is_successful(cdnskey.status);
}

class ScanResultsAppender
{
public:
    explicit ScanResultsAppender(ScanResultCollection::ScanResults& results)
        : results_{results}
    { }
    void operator()(const Unwrapper::ScanResult& unwrapper) const
    {
        results_.push_back({
                unwrapper.scan_job_id(),
                unwrapper.scan_at(),
                unwrapper.has_nameserver_ip() ? IpAddress::Optional{unwrapper.nameserver_ip()}
                                              : IpAddress::nullopt,
                unwrapper.cdnskey(),
                unwrapper.has_nameserver() ? Nameserver::Optional{unwrapper.nameserver()}
                                           : Nameserver::nullopt,
        });
    }
private:
    ScanResultCollection::ScanResults& results_;
};

}//namespace CdnskeyProcessor

using namespace CdnskeyProcessor;

std::string CdnskeyProcessor::serialize(const ScanResultCollection& collection)
{
    Api::Internal::ScanResultCollection proto;
    Wrapper::make_wrapper(proto).set(collection);
    return proto.SerializeAsString();
}

template <>
ScanResultCollection CdnskeyProcessor::deserialize(const char* msg, std::size_t size)
{
    Api::Internal::ScanResultCollection proto;
    proto.ParseFromArray(msg, size);
    const auto unwrapper = Unwrapper::make_unwrapper(proto);
    auto result = ScanResultCollection
    {
        unwrapper.scan_batch_id(),
        unwrapper.worker_name(),
        unwrapper.sent_at(),
        {unwrapper.has_started_at(), [&]() { return unwrapper.started_at(); }},
        {unwrapper.has_finished_at(), [&]() { return unwrapper.finished_at(); }},
        {}
    };
    result.scan_results.reserve(unwrapper.items_size());
    unwrapper.items(ScanResultsAppender{result.scan_results});
    return result;
}
