/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SUBPROCESS_HH_6A96B65C68F846685307A32041A4A7FD
#define SUBPROCESS_HH_6A96B65C68F846685307A32041A4A7FD

#include <sys/wait.h>
#include <unistd.h>

#include <array>
#include <exception>
#include <string>
#include <vector>

namespace CdnskeyProcessor {

struct SubprocessException : std::exception { };
struct SubprocessChildException : std::exception { };

class Subprocess
{
public:
    using ReadBuffer = std::array<char, 1024>;

    explicit Subprocess(const std::vector<const char*>& _argv);
    Subprocess(const Subprocess&) = delete;
    Subprocess(Subprocess&&) = delete;
    Subprocess& operator=(const Subprocess&) = delete;
    Subprocess& operator=(Subprocess&&) = delete;

    pid_t parent_pid() const;
    pid_t child_pid() const;

    int wait();
    ssize_t write(const std::string&);
    void write_eof();
    ssize_t read(ReadBuffer&);
private:
    class Pipe
    {
    public:
        Pipe();
        ~Pipe();
        int read_fd() const;
        int write_fd() const;
        void close();
    private:
        int fd_[2];
    };

    Pipe write_pipe_;
    Pipe read_pipe_;
    pid_t parent_pid_ = -1;
    pid_t child_pid_ = -1;
};

} // namespace CdnskeyProcessor

#endif//SUBPROCESS_HH_6A96B65C68F846685307A32041A4A7FD
