/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef TIMER_HH_43BDBCCE6FC6883EBF5EEA32B78A4C52
#define TIMER_HH_43BDBCCE6FC6883EBF5EEA32B78A4C52

#include <boost/asio/io_context.hpp>

#include <chrono>
#include <functional>
#include <list>
#include <map>

namespace CdnskeyProcessor {
namespace Internal {

class Timer
{
public:
    explicit Timer(int concurrency_hint);
    ~Timer();
    using TimePoint = std::chrono::steady_clock::time_point;
    using Callback = std::function<void(TimePoint)>;
    using CallbackList = std::list<Callback>;
    struct CallbackHandle
    {
        TimePoint scheduled;
        CallbackList::const_iterator callback_list_iter;
    };
    CallbackHandle register_callback(TimePoint scheduled, Callback callback);
    void erase_callback(const CallbackHandle& callback_handle);
    void loop();
    boost::asio::io_context& service();
private:
    using CallbacksByTime = std::map<TimePoint, CallbackList>;
    boost::asio::io_context service_;
    CallbacksByTime scheduler_;
};

} // namespace CdnskeyProcessor::Internal
} // namespace CdnskeyProcessor

#endif//TIMER_HH_43BDBCCE6FC6883EBF5EEA32B78A4C52
