/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SCAN_TASK_HH_B800DEFE4ACF6D1EE133258AC1C67CC5//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define SCAN_TASK_HH_B800DEFE4ACF6D1EE133258AC1C67CC5

#include "src/common_types.hh"

#include <cstdint>
#include <map>
#include <set>
#include <string>
#include <vector>

namespace CdnskeyProcessor {

using NumberOfQueries = LibStrong::Type<std::size_t, struct NumberOfQueriesTag_>;

struct ScanTask
{
    DomainFqdn domain_fqdn;
    DomainUid domain_uid;
    std::set<Nameserver> nameservers;
};

struct ScanJob
{
    ScanJobId scan_job_id;
    DomainFqdn domain_fqdn;
    DomainUid domain_uid;
};

using SecureJobs = std::vector<ScanJob>;
using InsecureJobs = std::map<Nameserver, std::vector<ScanJob>>;

struct SecureScanBatch
{
    ScanBatchId batch_id;
    SecureJobs jobs;
};
std::string serialize(const SecureScanBatch&);
NumberOfQueries estimated_size(const SecureScanBatch&);

struct InsecureScanBatch
{
    ScanBatchId batch_id;
    InsecureJobs jobs;
};
std::string serialize(const InsecureScanBatch&);
NumberOfQueries estimated_size(const InsecureScanBatch&);

using CdnskeyFlags = LibStrong::Type<std::uint32_t, struct CdnskeyFlagsTag_>;
using CdnskeyProto = LibStrong::Type<std::uint32_t, struct CdnskeyProtoTag_>;
using CdnskeyAlg = LibStrong::Type<std::uint32_t, struct CdnskeyAlgTag_>;
using CdnskeyPublicKey = LibStrong::Type<std::string, struct CdnskeyPublicKeyTag_>;

enum class CdnskeyStatus
{
    insecure,
    insecure_empty,
    secure,
    secure_empty,
    untrustworthy,
    unknown,
    unresolved,
    unresolved_ip
};

bool is_successful(CdnskeyStatus);
bool does_contain_result(CdnskeyStatus);

struct Cdnskey
{
    CdnskeyStatus status;
    CdnskeyFlags::Optional flags = CdnskeyFlags::nullopt;
    CdnskeyProto::Optional proto = CdnskeyProto::nullopt;
    CdnskeyAlg::Optional alg = CdnskeyAlg::nullopt;
    CdnskeyPublicKey::Optional public_key = CdnskeyPublicKey::nullopt;
};

template <typename T>
T deserialize(const char*, std::size_t);

template <>
InsecureScanBatch deserialize(const char*, std::size_t);

template <>
SecureScanBatch deserialize(const char*, std::size_t);

}//namespace CdnskeyProcessor

#endif//SCAN_TASK_HH_B800DEFE4ACF6D1EE133258AC1C67CC5
