/*
 * Copyright (C) 2021-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef STORAGE_HH_51D79DACF29358621CFEAD41425C8D19//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define STORAGE_HH_51D79DACF29358621CFEAD41425C8D19

#include "src/common_types.hh"
#include "src/database.hh"
#include "src/domain_cdnskey.hh"
#include "src/internal/scan_result.hh"
#include "src/raw_scan_results.hh"
#include "src/scan_task.hh"

#include <libpg/pg_rw_transaction.hh>

#include <exception>
#include <set>

namespace CdnskeyProcessor {

ScanQueueId open_scan_queue(const LibPg::PgRwTransaction& tx);
void close_scan_queue(const LibPg::PgRwTransaction& tx, ScanQueueId);
std::size_t split_into_batches(const LibPg::PgRwTransaction& tx, ScanQueueId queue_id, std::size_t batch_size);

struct LessByDomainFqdn
{
    bool operator()(const ScanTask&, const ScanTask&) const noexcept;
};

using ScanTasksSet = std::set<ScanTask, LessByDomainFqdn>;

void add_scan_tasks(const LibPg::PgRwTransaction& tx, ScanQueueId scan_queue_id, const ScanTasksSet& scan_tasks);

bool add_worker(const LibPg::PgRwTransaction& tx, const WorkerName& worker_name);
bool remove_worker(const LibPg::PgRwTransaction& tx, const WorkerName& worker_name);

SecureScanBatch get_secure_batch_to_solve(const LibPg::PgTransaction& tx);
InsecureScanBatch get_insecure_batch_to_solve(const LibPg::PgTransaction& tx);

void batch_sent(const LibPg::PgRwTransaction& tx, ScanBatchId);
void batch_started(const LibPg::PgRwTransaction& tx, ScanBatchId);
void batch_started(const LibPg::PgRwTransaction& tx, ScanBatchId, const WorkerName&);
void batch_result_received(const LibPg::PgRwTransaction& tx, const ScanResultCollection&);

AcceptableCdnskeys get_acceptable_cdnskey_records(
        const LibPg::PgTransaction& tx,
        MaxScanPeriod max_scan_period,
        MinPeriodFixedCdnskey min_period_fixed_cdnskey);

struct InvalidArgument : std::exception
{
    const char* what() const noexcept override = 0;
};

Report::RawScanResultsReply get_raw_scan_results(
        const LibPg::PgTransaction& tx,
        const DomainFqdn& domain);

Report::RawScanResultsReply get_raw_scan_results(
        const LibPg::PgTransaction& tx,
        const DomainUid& domain);

void clean_old_records(
        const LibPg::PgRwTransaction& tx,
        MaxScanPeriod max_scan_period,
        MinPeriodFixedCdnskey min_period_fixed_cdnskey);

//VACUUM cannot run inside a transaction block
void vacuum(const LibPg::PgConnection& conn);

}//namespace CdnskeyProcessor

#endif//STORAGE_HH_51D79DACF29358621CFEAD41425C8D19
