/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef UNWRAPPER_HH_2C692125DAF1B73BDBFBCF329BD02738//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define UNWRAPPER_HH_2C692125DAF1B73BDBFBCF329BD02738

namespace CdnskeyProcessor {

template <typename Unwrapper>
class ScanTaskCollectionRequestUnwrapper
{
public:
    template <typename Visitor>
    decltype(auto) items(const Visitor& visitor) const
    {
        return (static_cast<const Unwrapper*>(this))->items(visitor);
    }
    decltype(auto) items_size() const
    {
        return (static_cast<const Unwrapper*>(this))->items_size();
    }
};

template <typename Unwrapper>
class ScanTaskUnwrapper
{
public:
    bool has_domain() const
    {
        return (static_cast<const Unwrapper*>(this))->has_domain();
    }
    decltype(auto) domain() const
    {
        return (static_cast<const Unwrapper*>(this))->domain();
    }
    template <typename Visitor>
    decltype(auto) nameservers(const Visitor& visitor) const
    {
        return (static_cast<const Unwrapper*>(this))->nameservers(visitor);
    }
    decltype(auto) nameservers_size() const
    {
        return (static_cast<const Unwrapper*>(this))->nameservers_size();
    }
};

template <typename Unwrapper>
class DomainUnwrapper
{
public:
    bool has_uid() const
    {
        return (static_cast<const Unwrapper*>(this))->has_uid();
    }
    decltype(auto) uid() const
    {
        return (static_cast<const Unwrapper*>(this))->uid();
    }
    bool has_fqdn() const
    {
        return (static_cast<const Unwrapper*>(this))->has_fqdn();
    }
    decltype(auto) fqdn() const
    {
        return (static_cast<const Unwrapper*>(this))->fqdn();
    }
    decltype(auto) scan_type() const
    {
        return (static_cast<const Unwrapper*>(this))->scan_type();
    }
};

template <typename Unwrapper>
class DomainUidUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class DomainFqdnUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class NameserverUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class ScanJobIdUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class ScanJobUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class InsecureJobsUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class SecureJobsUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class AddWorkerRequestUnwrapper
{
public:
    bool has_worker_name() const
    {
        return (static_cast<const Unwrapper*>(this))->has_worker_name();
    }
    decltype(auto) worker_name() const
    {
        return (static_cast<const Unwrapper*>(this))->worker_name();
    }
};

template <typename Unwrapper>
class RemoveWorkerRequestUnwrapper
{
public:
    bool has_worker_name() const
    {
        return (static_cast<const Unwrapper*>(this))->has_worker_name();
    }
    decltype(auto) worker_name() const
    {
        return (static_cast<const Unwrapper*>(this))->worker_name();
    }
};

template <typename Unwrapper>
class WorkerNameUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class ScanBatchIdUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class InsecureScanBatchUnwrapper
{
public:
    bool has_id() const
    {
        return (static_cast<const Unwrapper*>(this))->has_id();
    }
    decltype(auto) id() const
    {
        return (static_cast<const Unwrapper*>(this))->id();
    }
    bool has_scan_tasks() const
    {
        return (static_cast<const Unwrapper*>(this))->has_scan_tasks();
    }
    decltype(auto) scan_tasks() const
    {
        return (static_cast<const Unwrapper*>(this))->scan_tasks();
    }
};

template <typename Unwrapper>
class SecureScanBatchUnwrapper
{
public:
    bool has_id() const
    {
        return (static_cast<const Unwrapper*>(this))->has_id();
    }
    decltype(auto) id() const
    {
        return (static_cast<const Unwrapper*>(this))->id();
    }
    bool has_scan_tasks() const
    {
        return (static_cast<const Unwrapper*>(this))->has_scan_tasks();
    }
    decltype(auto) scan_tasks() const
    {
        return (static_cast<const Unwrapper*>(this))->scan_tasks();
    }
};

template <typename Unwrapper>
class DnskeyFlagsUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class DnskeyProtoUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class DnskeyAlgUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class DnskeyPublicKeyUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class IpAddressUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class TimestampUnwrapper
{
public:
    decltype(auto) operator*() const
    {
        return **static_cast<const Unwrapper*>(this);
    }
};

template <typename Unwrapper>
class CdnskeyUnwrapper
{
public:
    decltype(auto) status() const
    {
        return static_cast<const Unwrapper*>(this)->status();
    }
    decltype(auto) flags() const
    {
        return static_cast<const Unwrapper*>(this)->flags();
    }
    bool has_flags() const
    {
        return static_cast<const Unwrapper*>(this)->has_flags();
    }
    decltype(auto) proto() const
    {
        return static_cast<const Unwrapper*>(this)->proto();
    }
    bool has_proto() const
    {
        return static_cast<const Unwrapper*>(this)->has_proto();
    }
    decltype(auto) alg() const
    {
        return static_cast<const Unwrapper*>(this)->alg();
    }
    bool has_alg() const
    {
        return static_cast<const Unwrapper*>(this)->has_alg();
    }
    decltype(auto) public_key() const
    {
        return static_cast<const Unwrapper*>(this)->public_key();
    }
    bool has_public_key() const
    {
        return static_cast<const Unwrapper*>(this)->has_public_key();
    }
};

template <typename Unwrapper>
class ScanResultUnwrapper
{
public:
    decltype(auto) scan_job_id() const
    {
        return static_cast<const Unwrapper*>(this)->scan_job_id();
    }
    bool has_scan_job_id() const
    {
        return static_cast<const Unwrapper*>(this)->has_scan_job_id();
    }
    decltype(auto) scan_at() const
    {
        return static_cast<const Unwrapper*>(this)->scan_at();
    }
    bool has_scan_at() const
    {
        return static_cast<const Unwrapper*>(this)->has_scan_at();
    }
    decltype(auto) nameserver_ip() const
    {
        return static_cast<const Unwrapper*>(this)->nameserver_ip();
    }
    bool has_nameserver_ip() const
    {
        return static_cast<const Unwrapper*>(this)->has_nameserver_ip();
    }
    decltype(auto) cdnskey() const
    {
        return static_cast<const Unwrapper*>(this)->cdnskey();
    }
    bool has_cdnskey() const
    {
        return static_cast<const Unwrapper*>(this)->has_cdnskey();
    }
    decltype(auto) nameserver() const
    {
        return static_cast<const Unwrapper*>(this)->nameserver();
    }
    bool has_nameserver() const
    {
        return static_cast<const Unwrapper*>(this)->has_nameserver();
    }
};

template <typename Unwrapper>
class ScanResultCollectionUnwrapper
{
public:
    decltype(auto) scan_batch_id() const
    {
        return static_cast<const Unwrapper*>(this)->scan_batch_id();
    }
    bool has_scan_batch_id() const
    {
        return static_cast<const Unwrapper*>(this)->has_scan_batch_id();
    }
    decltype(auto) worker_name() const
    {
        return static_cast<const Unwrapper*>(this)->worker_name();
    }
    bool has_worker_name() const
    {
        return static_cast<const Unwrapper*>(this)->has_worker_name();
    }
    decltype(auto) sent_at() const
    {
        return static_cast<const Unwrapper*>(this)->sent_at();
    }
    bool has_sent_at() const
    {
        return static_cast<const Unwrapper*>(this)->has_sent_at();
    }
    decltype(auto) started_at() const
    {
        return static_cast<const Unwrapper*>(this)->started_at();
    }
    bool has_started_at() const
    {
        return static_cast<const Unwrapper*>(this)->has_started_at();
    }
    decltype(auto) finished_at() const
    {
        return static_cast<const Unwrapper*>(this)->finished_at();
    }
    bool has_finished_at() const
    {
        return static_cast<const Unwrapper*>(this)->has_finished_at();
    }
    template <typename Visitor>
    decltype(auto) items(const Visitor& visitor) const
    {
        return (static_cast<const Unwrapper*>(this))->items(visitor);
    }
    decltype(auto) items_size() const
    {
        return (static_cast<const Unwrapper*>(this))->items_size();
    }
};

template <typename Unwrapper>
class RawScanResultsRequestUnwrapper
{
public:
    decltype(auto) domain_uid() const
    {
        return static_cast<const Unwrapper*>(this)->domain_uid();
    }
    bool has_domain_uid() const
    {
        return static_cast<const Unwrapper*>(this)->has_domain_uid();
    }
    decltype(auto) domain_fqdn() const
    {
        return static_cast<const Unwrapper*>(this)->domain_fqdn();
    }
    bool has_domain_fqdn() const
    {
        return static_cast<const Unwrapper*>(this)->has_domain_fqdn();
    }
};

}//namespace CdnskeyProcessor

#endif//UNWRAPPER_HH_2C692125DAF1B73BDBFBCF329BD02738
