/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef UNWRAPPER_HH_3BA27C3A5A0C8EB2A8BFB06B20BF26FF//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define UNWRAPPER_HH_3BA27C3A5A0C8EB2A8BFB06B20BF26FF

#include "src/common_types.hh"
#include "src/scan_task.hh"
#include "src/unwrapper.hh"

#include "cdnskey_processor_api/service_report_grpc.pb.h"
#include "cdnskey_processor_api/service_supervisor_grpc.pb.h"
#include "protos/scan_result.pb.h"
#include "protos/scan_task.pb.h"

#include <algorithm>

namespace CdnskeyProcessor {
namespace Unwrapper {

class DomainUid : public DomainUidUnwrapper<DomainUid>
{
public:
    CdnskeyProcessor::DomainUid operator*() const;
private:
    explicit DomainUid(const Api::DomainUid&);
    const Api::DomainUid& data_;
    friend DomainUid make_unwrapper(const Api::DomainUid&);
};
DomainUid make_unwrapper(const Api::DomainUid&);

class DomainFqdn : public DomainFqdnUnwrapper<DomainFqdn>
{
public:
    CdnskeyProcessor::DomainFqdn operator*() const;
private:
    explicit DomainFqdn(const Api::DomainFqdn&);
    const Api::DomainFqdn& data_;
    friend DomainFqdn make_unwrapper(const Api::DomainFqdn&);
};
DomainFqdn make_unwrapper(const Api::DomainFqdn&);

class Nameserver : public NameserverUnwrapper<Nameserver>
{
public:
    CdnskeyProcessor::Nameserver operator*() const;
private:
    explicit Nameserver(const Api::Nameserver&);
    const Api::Nameserver& data_;
    friend Nameserver make_unwrapper(const Api::Nameserver&);
};
Nameserver make_unwrapper(const Api::Nameserver&);

class ScanJobId : public ScanJobIdUnwrapper<ScanJobId>
{
public:
    CdnskeyProcessor::ScanJobId operator*() const;
private:
    explicit ScanJobId(const Api::Internal::ScanJobId&);
    const Api::Internal::ScanJobId& data_;
    friend ScanJobId make_unwrapper(const Api::Internal::ScanJobId&);
};
ScanJobId make_unwrapper(const Api::Internal::ScanJobId&);

class ScanJob : public ScanJobUnwrapper<ScanJob>
{
public:
    CdnskeyProcessor::ScanJob operator*() const;
private:
    explicit ScanJob(const Api::Internal::ScanJob&);
    const Api::Internal::ScanJob& data_;
    friend ScanJob make_unwrapper(const Api::Internal::ScanJob&);
};
ScanJob make_unwrapper(const Api::Internal::ScanJob&);

class InsecureJobs : public InsecureJobsUnwrapper<InsecureJobs>
{
public:
    CdnskeyProcessor::InsecureJobs operator*() const;
private:
    explicit InsecureJobs(const Api::Internal::InsecureScanTaskCollection&);
    const Api::Internal::InsecureScanTaskCollection& data_;
    friend InsecureJobs make_unwrapper(const Api::Internal::InsecureScanTaskCollection&);
};
InsecureJobs make_unwrapper(const Api::Internal::InsecureScanTaskCollection&);

class SecureJobs : public SecureJobsUnwrapper<SecureJobs>
{
public:
    CdnskeyProcessor::SecureJobs operator*() const;
private:
    explicit SecureJobs(const Api::Internal::SecureScanTaskCollection&);
    const Api::Internal::SecureScanTaskCollection& data_;
    friend SecureJobs make_unwrapper(const Api::Internal::SecureScanTaskCollection&);
};
SecureJobs make_unwrapper(const Api::Internal::SecureScanTaskCollection&);

class WorkerName : public WorkerNameUnwrapper<WorkerName>
{
public:
    CdnskeyProcessor::WorkerName operator*() const;
private:
    explicit WorkerName(const Api::WorkerName&);
    const Api::WorkerName& data_;
    friend WorkerName make_unwrapper(const Api::WorkerName&);
};
WorkerName make_unwrapper(const Api::WorkerName&);

class Domain : public DomainUnwrapper<Domain>
{
public:
    bool has_uid() const;
    CdnskeyProcessor::DomainUid uid() const;
    bool has_fqdn() const;
    CdnskeyProcessor::DomainFqdn fqdn() const;
    ScanType scan_type() const;
private:
    explicit Domain(const Api::Domain&);
    const Api::Domain& data_;
    friend Domain make_unwrapper(const Api::Domain&);
};
Domain make_unwrapper(const Api::Domain&);

class ScanTask : public ScanTaskUnwrapper<ScanTask>
{
public:
    bool has_domain() const;
    Domain domain() const;
    template <typename Visitor>
    decltype(auto) nameservers(const Visitor& visitor) const
    {
        std::for_each(data_.nameservers().begin(), data_.nameservers().end(), [&visitor](auto&& nameserver)
        {
            visitor(*make_unwrapper(nameserver));
        });
        return data_.nameservers_size();
    }
    std::size_t nameservers_size() const;
private:
    explicit ScanTask(const Api::Supervisor::ScanTask& data);
    const Api::Supervisor::ScanTask& data_;
    friend ScanTask make_unwrapper(const Api::Supervisor::ScanTask&);
};
ScanTask make_unwrapper(const Api::Supervisor::ScanTask&);

class ScanTaskCollectionRequest : public ScanTaskCollectionRequestUnwrapper<ScanTaskCollectionRequest>
{
public:
    template <typename Visitor>
    decltype(auto) items(const Visitor& visitor) const
    {
        std::for_each(data_.items().begin(), data_.items().end(), [&visitor](auto&& item)
        {
            visitor(make_unwrapper(item));
        });
        return data_.items_size();
    }
    std::size_t items_size() const;
private:
    explicit ScanTaskCollectionRequest(const Api::Supervisor::ScanTaskCollectionRequest& data);
    const Api::Supervisor::ScanTaskCollectionRequest& data_;
    friend ScanTaskCollectionRequest make_unwrapper(const Api::Supervisor::ScanTaskCollectionRequest&);
};
ScanTaskCollectionRequest make_unwrapper(const Api::Supervisor::ScanTaskCollectionRequest&);

class AddWorkerRequest : public AddWorkerRequestUnwrapper<AddWorkerRequest>
{
public:
    bool has_worker_name() const;
    CdnskeyProcessor::WorkerName worker_name() const;
private:
    explicit AddWorkerRequest(const Api::Supervisor::AddWorkerRequest& data);
    const Api::Supervisor::AddWorkerRequest& data_;
    friend AddWorkerRequest make_unwrapper(const Api::Supervisor::AddWorkerRequest&);
};
AddWorkerRequest make_unwrapper(const Api::Supervisor::AddWorkerRequest&);

class RemoveWorkerRequest : public RemoveWorkerRequestUnwrapper<RemoveWorkerRequest>
{
public:
    bool has_worker_name() const;
    CdnskeyProcessor::WorkerName worker_name() const;
private:
    explicit RemoveWorkerRequest(const Api::Supervisor::RemoveWorkerRequest& data);
    const Api::Supervisor::RemoveWorkerRequest& data_;
    friend RemoveWorkerRequest make_unwrapper(const Api::Supervisor::RemoveWorkerRequest&);
};
RemoveWorkerRequest make_unwrapper(const Api::Supervisor::RemoveWorkerRequest&);

class ScanBatchId : public ScanBatchIdUnwrapper<ScanBatchId>
{
public:
    CdnskeyProcessor::ScanBatchId operator*() const;
private:
    explicit ScanBatchId(const Api::Internal::ScanBatchId&);
    const Api::Internal::ScanBatchId& data_;
    friend ScanBatchId make_unwrapper(const Api::Internal::ScanBatchId&);
};
ScanBatchId make_unwrapper(const Api::Internal::ScanBatchId&);

class InsecureScanBatch : InsecureScanBatchUnwrapper<InsecureScanBatch>
{
public:
    bool has_id() const;
    CdnskeyProcessor::ScanBatchId id() const;
    bool has_scan_tasks() const;
    CdnskeyProcessor::InsecureJobs scan_tasks() const;
private:
    explicit InsecureScanBatch(const Api::Internal::ScanBatchInsecure& data);
    const Api::Internal::ScanBatchInsecure& data_;
    friend InsecureScanBatch make_unwrapper(const Api::Internal::ScanBatchInsecure&);
};
InsecureScanBatch make_unwrapper(const Api::Internal::ScanBatchInsecure&);

class SecureScanBatch : SecureScanBatchUnwrapper<SecureScanBatch>
{
public:
    bool has_id() const;
    CdnskeyProcessor::ScanBatchId id() const;
    bool has_scan_tasks() const;
    CdnskeyProcessor::SecureJobs scan_tasks() const;
private:
    explicit SecureScanBatch(const Api::Internal::ScanBatchSecure& data);
    const Api::Internal::ScanBatchSecure& data_;
    friend SecureScanBatch make_unwrapper(const Api::Internal::ScanBatchSecure&);
};
SecureScanBatch make_unwrapper(const Api::Internal::ScanBatchSecure&);

class DnskeyFlags : public DnskeyFlagsUnwrapper<DnskeyFlags>
{
public:
    CdnskeyProcessor::CdnskeyFlags operator*() const;
private:
    explicit DnskeyFlags(const Api::DnskeyFlags&);
    const Api::DnskeyFlags& data_;
    friend DnskeyFlags make_unwrapper(const Api::DnskeyFlags&);
};
DnskeyFlags make_unwrapper(const Api::DnskeyFlags&);

class DnskeyProto : public DnskeyProtoUnwrapper<DnskeyProto>
{
public:
    CdnskeyProcessor::CdnskeyProto operator*() const;
private:
    explicit DnskeyProto(const Api::DnskeyProto&);
    const Api::DnskeyProto& data_;
    friend DnskeyProto make_unwrapper(const Api::DnskeyProto&);
};
DnskeyProto make_unwrapper(const Api::DnskeyProto&);

class DnskeyAlg : public DnskeyAlgUnwrapper<DnskeyAlg>
{
public:
    CdnskeyProcessor::CdnskeyAlg operator*() const;
private:
    explicit DnskeyAlg(const Api::DnskeyAlg&);
    const Api::DnskeyAlg& data_;
    friend DnskeyAlg make_unwrapper(const Api::DnskeyAlg&);
};
DnskeyAlg make_unwrapper(const Api::DnskeyAlg&);

class DnskeyPublicKey : public DnskeyPublicKeyUnwrapper<DnskeyPublicKey>
{
public:
    CdnskeyProcessor::CdnskeyPublicKey operator*() const;
private:
    explicit DnskeyPublicKey(const Api::DnskeyPublicKey&);
    const Api::DnskeyPublicKey& data_;
    friend DnskeyPublicKey make_unwrapper(const Api::DnskeyPublicKey&);
};
DnskeyPublicKey make_unwrapper(const Api::DnskeyPublicKey&);

class IpAddress : public IpAddressUnwrapper<IpAddress>
{
public:
    CdnskeyProcessor::IpAddress operator*() const;
private:
    explicit IpAddress(const Api::IpAddress&);
    const Api::IpAddress& data_;
    friend IpAddress make_unwrapper(const Api::IpAddress&);
};
IpAddress make_unwrapper(const Api::IpAddress&);

class Timestamp : public TimestampUnwrapper<Timestamp>
{
public:
    CdnskeyProcessor::Timestamp operator*() const;
private:
    explicit Timestamp(const google::protobuf::Timestamp&);
    const google::protobuf::Timestamp& data_;
    friend Timestamp make_unwrapper(const google::protobuf::Timestamp&);
};
Timestamp make_unwrapper(const google::protobuf::Timestamp&);

class Cdnskey : public CdnskeyUnwrapper<Cdnskey>
{
public:
    CdnskeyProcessor::CdnskeyStatus status() const;
    CdnskeyProcessor::CdnskeyFlags flags() const;
    bool has_flags() const;
    CdnskeyProcessor::CdnskeyProto proto() const;
    bool has_proto() const;
    CdnskeyProcessor::CdnskeyAlg alg() const;
    bool has_alg() const;
    CdnskeyProcessor::CdnskeyPublicKey public_key() const;
    bool has_public_key() const;
private:
    explicit Cdnskey(const Api::Cdnskey&);
    const Api::Cdnskey& data_;
    friend Cdnskey make_unwrapper(const Api::Cdnskey&);
};
Cdnskey make_unwrapper(const Api::Cdnskey&);

class ScanResult : public ScanResultUnwrapper<ScanResult>
{
public:
    CdnskeyProcessor::ScanJobId scan_job_id() const;
    bool has_scan_job_id() const;
    CdnskeyProcessor::Timestamp scan_at() const;
    bool has_scan_at() const;
    CdnskeyProcessor::IpAddress nameserver_ip() const;
    bool has_nameserver_ip() const;
    CdnskeyProcessor::Cdnskey cdnskey() const;
    bool has_cdnskey() const;
    CdnskeyProcessor::Nameserver nameserver() const;
    bool has_nameserver() const;
private:
    explicit ScanResult(const Api::Internal::ScanResult&);
    const Api::Internal::ScanResult& data_;
    friend ScanResult make_unwrapper(const Api::Internal::ScanResult&);
};
ScanResult make_unwrapper(const Api::Internal::ScanResult&);

class ScanResultCollection : public ScanResultCollectionUnwrapper<ScanResultCollection>
{
public:
    CdnskeyProcessor::ScanBatchId scan_batch_id() const;
    bool has_scan_batch_id() const;
    CdnskeyProcessor::WorkerName worker_name() const;
    bool has_worker_name() const;
    CdnskeyProcessor::Timestamp sent_at() const;
    bool has_sent_at() const;
    CdnskeyProcessor::Timestamp started_at() const;
    bool has_started_at() const;
    CdnskeyProcessor::Timestamp finished_at() const;
    bool has_finished_at() const;
    template <typename Visitor>
    void items(const Visitor& visitor) const
    {
        std::for_each(data_.items().begin(), data_.items().end(), [&visitor](auto&& item)
        {
            visitor(make_unwrapper(item));
        });
    }
    std::size_t items_size() const;
private:
    explicit ScanResultCollection(const Api::Internal::ScanResultCollection&);
    const Api::Internal::ScanResultCollection& data_;
    friend ScanResultCollection make_unwrapper(const Api::Internal::ScanResultCollection&);
};
ScanResultCollection make_unwrapper(const Api::Internal::ScanResultCollection&);

class RawScanResultsRequest : public RawScanResultsRequestUnwrapper<RawScanResultsRequest>
{
public:
    CdnskeyProcessor::DomainUid domain_uid() const;
    bool has_domain_uid() const;
    CdnskeyProcessor::DomainFqdn domain_fqdn() const;
    bool has_domain_fqdn() const;
private:
    explicit RawScanResultsRequest(const Api::Report::RawScanResultsRequest&);
    const Api::Report::RawScanResultsRequest& data_;
    friend RawScanResultsRequest make_unwrapper(const Api::Report::RawScanResultsRequest&);
};
RawScanResultsRequest make_unwrapper(const Api::Report::RawScanResultsRequest&);

}//namespace CdnskeyProcessor::Unwrapper
}//namespace CdnskeyProcessor

#endif//UNWRAPPER_HH_3BA27C3A5A0C8EB2A8BFB06B20BF26FF
