/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef WRAPPER_HH_1FF783CA7C517F7871C575B25523992E//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define WRAPPER_HH_1FF783CA7C517F7871C575B25523992E

#include "src/common_types.hh"
#include "src/internal/scan_result.hh"
#include "src/raw_scan_results.hh"
#include "src/scan_task.hh"

#include <cstdint>
#include <string>
#include <utility>

namespace CdnskeyProcessor {

template <typename T>
class ScanBatchIdWrapper
{
public:
    ScanBatchIdWrapper& set(ScanBatchId id)
    {
        static_cast<T*>(this)->set(id);
        return *this;
    }
};

template <typename T>
class ScanJobIdWrapper
{
public:
    ScanJobIdWrapper& set(ScanJobId id)
    {
        static_cast<T*>(this)->set(id);
        return *this;
    }
};

template <typename T>
class DomainFqdnWrapper
{
public:
    DomainFqdnWrapper& set(DomainFqdn fqdn)
    {
        static_cast<T*>(this)->set(std::move(fqdn));
        return *this;
    }
};

template <typename T>
class DomainUidWrapper
{
public:
    DomainUidWrapper& set(DomainUid uid)
    {
        static_cast<T*>(this)->set(std::move(uid));
        return *this;
    }
};

template <typename T>
class DomainWrapper
{
public:
    DomainWrapper& set_uid(DomainUid uid)
    {
        static_cast<T*>(this)->set_uid(std::move(uid));
        return *this;
    }
    DomainWrapper& set_fqdn(DomainFqdn fqdn)
    {
        static_cast<T*>(this)->set_fqdn(std::move(fqdn));
        return *this;
    }
    DomainWrapper& set_scan_type(ScanType type)
    {
        static_cast<T*>(this)->set_scan_type(type);
        return *this;
    }
};

template <typename T>
class ScanJobWrapper
{
public:
    ScanJobWrapper& set_domain(DomainUid uid, DomainFqdn fqdn, ScanType type)
    {
        static_cast<T*>(this)->set_domain(std::move(uid), std::move(fqdn), type);
        return *this;
    }
    ScanJobWrapper& set_id(ScanJobId id)
    {
        static_cast<T*>(this)->set_id(id);
        return *this;
    }
};

template <typename T>
class NameserverWrapper
{
public:
    NameserverWrapper& set(Nameserver nameserver)
    {
        static_cast<T*>(this)->set(std::move(nameserver));
        return *this;
    }
};

template <typename T>
class WorkerNameWrapper
{
public:
    WorkerNameWrapper& set(WorkerName worker)
    {
        static_cast<T*>(this)->set(std::move(worker));
        return *this;
    }
};

template <typename T>
class InsecureScanTaskCollectionWrapper
{
public:
    template <typename Iter>
    InsecureScanTaskCollectionWrapper& set_items(Iter&& begin, Iter&& end)
    {
        static_cast<T*>(this)->set_items(std::forward<Iter>(begin), std::forward<Iter>(end));
        return *this;
    }
};

template <typename T>
class SecureScanTaskCollectionWrapper
{
public:
    template <typename Iter>
    SecureScanTaskCollectionWrapper& set_items(Iter&& begin, Iter&& end)
    {
        static_cast<T*>(this)->set_items(std::forward<Iter>(begin), std::forward<Iter>(end));
        return *this;
    }
};

template <typename T>
class ScanBatchInsecureWrapper
{
public:
    ScanBatchInsecureWrapper& set(const InsecureScanBatch& batch)
    {
        static_cast<T*>(this)->set(batch);
        return *this;
    }
};

template <typename T>
class ScanBatchSecureWrapper
{
public:
    ScanBatchSecureWrapper& set(const SecureScanBatch& batch)
    {
        static_cast<T*>(this)->set(batch);
        return *this;
    }
};

template <typename T>
class TimestampWrapper
{
public:
    TimestampWrapper& set(const Timestamp& timestamp)
    {
        static_cast<T*>(this)->set(timestamp);
        return *this;
    }
};

template <typename T>
class IpAddressWrapper
{
public:
    IpAddressWrapper& set(const IpAddress& ip_address)
    {
        static_cast<T*>(this)->set(ip_address);
        return *this;
    }
};

template <typename T>
class CdnskeyFlagsWrapper
{
public:
    CdnskeyFlagsWrapper& set(const CdnskeyFlags& flags)
    {
        static_cast<T*>(this)->set(flags);
        return *this;
    }
};

template <typename T>
class CdnskeyProtoWrapper
{
public:
    CdnskeyProtoWrapper& set(const CdnskeyProto& proto)
    {
        static_cast<T*>(this)->set(proto);
        return *this;
    }
};

template <typename T>
class CdnskeyAlgWrapper
{
public:
    CdnskeyAlgWrapper& set(const CdnskeyAlg& alg)
    {
        static_cast<T*>(this)->set(alg);
        return *this;
    }
};

template <typename T>
class CdnskeyPublicKeyWrapper
{
public:
    CdnskeyPublicKeyWrapper& set(const CdnskeyPublicKey& key)
    {
        static_cast<T*>(this)->set(key);
        return *this;
    }
};

template <typename T>
class CdnskeyWrapper
{
public:
    CdnskeyWrapper& set(const Cdnskey& alg)
    {
        static_cast<T*>(this)->set(alg);
        return *this;
    }
};

template <typename T>
class ScanResultWrapper
{
public:
    ScanResultWrapper& set(const ScanResult& result)
    {
        static_cast<T*>(this)->set(result);
        return *this;
    }
};

template <typename T>
class ScanResultCollectionWrapper
{
public:
    ScanResultCollectionWrapper& set(const ScanResultCollection& collection)
    {
        static_cast<T*>(this)->set(collection);
        return *this;
    }
};

template <typename T>
class AcceptableDnskeysReplyWrapper
{
public:
    template <typename Iter>
    AcceptableDnskeysReplyWrapper& set_items(Iter&& begin, Iter&& end)
    {
        static_cast<T*>(this)->set_items(std::forward<Iter>(begin), std::forward<Iter>(end));
        return *this;
    }
};

template <typename T>
class RawScanResultsReplyWrapper
{
public:
    template <typename Iter>
    RawScanResultsReplyWrapper& set_items(Iter&& begin, Iter&& end)
    {
        static_cast<T*>(this)->set_items(std::forward<Iter>(begin), std::forward<Iter>(end));
        return *this;
    }
};

}//namespace CdnskeyProcessor

#endif//WRAPPER_HH_1FF783CA7C517F7871C575B25523992E
