/*
 * Copyright (C) 2023-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef WRAPPER_HH_BFA7226DF6F9ED86863B19DCC4F1DAE8//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define WRAPPER_HH_BFA7226DF6F9ED86863B19DCC4F1DAE8

#include "src/wrapper.hh"
#include "src/domain_cdnskey.hh"

#include "cdnskey_processor_api/service_report_grpc.pb.h"
#include "cdnskey_processor_api/service_supervisor_grpc.pb.h"

#include "protos/scan_result.pb.h"
#include "protos/scan_task.pb.h"

namespace CdnskeyProcessor {
namespace Wrapper {

class ScanBatchId : public ScanBatchIdWrapper<ScanBatchId>
{
public:
    void set(CdnskeyProcessor::ScanBatchId id);
private:
    explicit ScanBatchId(Api::Internal::ScanBatchId& data);
    Api::Internal::ScanBatchId& data_;
    friend ScanBatchId make_wrapper(Api::Internal::ScanBatchId& data);
};
ScanBatchId make_wrapper(Api::Internal::ScanBatchId& data);

class ScanJobId : public ScanJobIdWrapper<ScanJobId>
{
public:
    void set(CdnskeyProcessor::ScanJobId id);
private:
    explicit ScanJobId(Api::Internal::ScanJobId& data);
    Api::Internal::ScanJobId& data_;
    friend ScanJobId make_wrapper(Api::Internal::ScanJobId& data);
};
ScanJobId make_wrapper(Api::Internal::ScanJobId& data);

class DomainFqdn : public DomainFqdnWrapper<DomainFqdn>
{
public:
    void set(CdnskeyProcessor::DomainFqdn fqdn);
private:
    explicit DomainFqdn(Api::DomainFqdn& data);
    Api::DomainFqdn& data_;
    friend DomainFqdn make_wrapper(Api::DomainFqdn& data);
};
DomainFqdn make_wrapper(Api::DomainFqdn& data);

class DomainUid : public DomainFqdnWrapper<DomainUid>
{
public:
    void set(CdnskeyProcessor::DomainUid uid);
private:
    explicit DomainUid(Api::DomainUid& data);
    Api::DomainUid& data_;
    friend DomainUid make_wrapper(Api::DomainUid& data);
};
DomainUid make_wrapper(Api::DomainUid& data);

class Domain : public DomainWrapper<Domain>
{
public:
    void set_uid(CdnskeyProcessor::DomainUid uid);
    void set_fqdn(CdnskeyProcessor::DomainFqdn fqdn);
    void set_scan_type(CdnskeyProcessor::ScanType type);
private:
    explicit Domain(Api::Domain& data);
    Api::Domain& data_;
    friend Domain make_wrapper(Api::Domain& data);
};
Domain make_wrapper(Api::Domain& data);

class ScanJob : public ScanJobWrapper<ScanJob>
{
public:
    void set_domain(
            CdnskeyProcessor::DomainUid uid,
            CdnskeyProcessor::DomainFqdn fqdn,
            CdnskeyProcessor::ScanType type);
    void set_id(CdnskeyProcessor::ScanJobId id);
private:
    explicit ScanJob(Api::Internal::ScanJob& data);
    Api::Internal::ScanJob& data_;
    friend ScanJob make_wrapper(Api::Internal::ScanJob& data);
};
ScanJob make_wrapper(Api::Internal::ScanJob& data);

class Nameserver : public NameserverWrapper<Nameserver>
{
public:
    void set(CdnskeyProcessor::Nameserver nameserver);
private:
    explicit Nameserver(Api::Nameserver& data);
    Api::Nameserver& data_;
    friend Nameserver make_wrapper(Api::Nameserver& data);
};
Nameserver make_wrapper(Api::Nameserver& data);

class WorkerName : public WorkerNameWrapper<WorkerName>
{
public:
    void set(CdnskeyProcessor::WorkerName worker);
private:
    explicit WorkerName(Api::WorkerName& data);
    Api::WorkerName& data_;
    friend WorkerName make_wrapper(Api::WorkerName& data);
};
WorkerName make_wrapper(Api::WorkerName& data);

class InsecureScanTaskCollection : public InsecureScanTaskCollectionWrapper<InsecureScanTaskCollection>
{
public:
    using Iterator = InsecureJobs::const_iterator;
    void set_items(Iterator begin, Iterator end);
private:
    explicit InsecureScanTaskCollection(Api::Internal::InsecureScanTaskCollection& data);
    Api::Internal::InsecureScanTaskCollection& data_;
    friend InsecureScanTaskCollection make_wrapper(Api::Internal::InsecureScanTaskCollection& data);
};
InsecureScanTaskCollection make_wrapper(Api::Internal::InsecureScanTaskCollection& data);

class SecureScanTaskCollection : public SecureScanTaskCollectionWrapper<SecureScanTaskCollection>
{
public:
    using Iterator = SecureJobs::const_iterator;
    void set_items(Iterator begin, Iterator end);
private:
    explicit SecureScanTaskCollection(Api::Internal::SecureScanTaskCollection& data);
    Api::Internal::SecureScanTaskCollection& data_;
    friend SecureScanTaskCollection make_wrapper(Api::Internal::SecureScanTaskCollection& data);
};
SecureScanTaskCollection make_wrapper(Api::Internal::SecureScanTaskCollection& data);

class ScanBatchInsecure : public ScanBatchInsecureWrapper<ScanBatchInsecure>
{
public:
    void set(const CdnskeyProcessor::InsecureScanBatch& batch);
private:
    explicit ScanBatchInsecure(Api::Internal::ScanBatchInsecure& data);
    Api::Internal::ScanBatchInsecure& data_;
    friend ScanBatchInsecure make_wrapper(Api::Internal::ScanBatchInsecure& data);
};
ScanBatchInsecure make_wrapper(Api::Internal::ScanBatchInsecure& data);

class ScanBatchSecure : public ScanBatchSecureWrapper<ScanBatchSecure>
{
public:
    void set(const CdnskeyProcessor::SecureScanBatch& batch);
private:
    explicit ScanBatchSecure(Api::Internal::ScanBatchSecure& data);
    Api::Internal::ScanBatchSecure& data_;
    friend ScanBatchSecure make_wrapper(Api::Internal::ScanBatchSecure& data);
};
ScanBatchSecure make_wrapper(Api::Internal::ScanBatchSecure& data);

class Timestamp : public TimestampWrapper<Timestamp>
{
public:
    void set(const CdnskeyProcessor::Timestamp& timestamp);
private:
    explicit Timestamp(google::protobuf::Timestamp& data);
    google::protobuf::Timestamp& data_;
    friend Timestamp make_wrapper(google::protobuf::Timestamp& data);
};
Timestamp make_wrapper(google::protobuf::Timestamp& data);

class IpAddress : public IpAddressWrapper<IpAddress>
{
public:
    void set(const CdnskeyProcessor::IpAddress& ip_address);
private:
    explicit IpAddress(Api::IpAddress& data);
    Api::IpAddress& data_;
    friend IpAddress make_wrapper(Api::IpAddress& data);
};
IpAddress make_wrapper(Api::IpAddress& data);

class CdnskeyFlags : public CdnskeyFlagsWrapper<CdnskeyFlags>
{
public:
    void set(const CdnskeyProcessor::CdnskeyFlags& flags);
private:
    explicit CdnskeyFlags(Api::DnskeyFlags& data);
    Api::DnskeyFlags& data_;
    friend CdnskeyFlags make_wrapper(Api::DnskeyFlags& data);
};
CdnskeyFlags make_wrapper(Api::DnskeyFlags& data);

class CdnskeyProto : public CdnskeyProtoWrapper<CdnskeyProto>
{
public:
    void set(const CdnskeyProcessor::CdnskeyProto& proto);
private:
    explicit CdnskeyProto(Api::DnskeyProto& data);
    Api::DnskeyProto& data_;
    friend CdnskeyProto make_wrapper(Api::DnskeyProto& data);
};
CdnskeyProto make_wrapper(Api::DnskeyProto& data);

class CdnskeyAlg : public CdnskeyAlgWrapper<CdnskeyAlg>
{
public:
    void set(const CdnskeyProcessor::CdnskeyAlg& alg);
private:
    explicit CdnskeyAlg(Api::DnskeyAlg& data);
    Api::DnskeyAlg& data_;
    friend CdnskeyAlg make_wrapper(Api::DnskeyAlg& data);
};
CdnskeyAlg make_wrapper(Api::DnskeyAlg& data);

class CdnskeyPublicKey : public CdnskeyPublicKeyWrapper<CdnskeyPublicKey>
{
public:
    void set(const CdnskeyProcessor::CdnskeyPublicKey& key);
private:
    explicit CdnskeyPublicKey(Api::DnskeyPublicKey& data);
    Api::DnskeyPublicKey& data_;
    friend CdnskeyPublicKey make_wrapper(Api::DnskeyPublicKey& data);
};
CdnskeyPublicKey make_wrapper(Api::DnskeyPublicKey& data);

class Cdnskey : public CdnskeyWrapper<Cdnskey>
{
public:
    void set(const CdnskeyProcessor::Cdnskey& cdnskey);
private:
    explicit Cdnskey(Api::Cdnskey& data);
    Api::Cdnskey& data_;
    friend Cdnskey make_wrapper(Api::Cdnskey& data);
};
Cdnskey make_wrapper(Api::Cdnskey& data);

class ScanResult : public ScanResultCollectionWrapper<ScanResult>
{
public:
    void set(const CdnskeyProcessor::ScanResult& result);
private:
    explicit ScanResult(Api::Internal::ScanResult& data);
    Api::Internal::ScanResult& data_;
    friend ScanResult make_wrapper(Api::Internal::ScanResult& data);
};
ScanResult make_wrapper(Api::Internal::ScanResult& data);

class ScanResultCollection : public ScanResultCollectionWrapper<ScanResultCollection>
{
public:
    void set(const CdnskeyProcessor::ScanResultCollection& collection);
private:
    explicit ScanResultCollection(Api::Internal::ScanResultCollection& data);
    Api::Internal::ScanResultCollection& data_;
    friend ScanResultCollection make_wrapper(Api::Internal::ScanResultCollection& data);
};
ScanResultCollection make_wrapper(Api::Internal::ScanResultCollection& data);

class AcceptableDnskeysReply : public AcceptableDnskeysReplyWrapper<AcceptableDnskeysReply>
{
public:
    using Iterator = AcceptableCdnskeys::const_iterator;
    void set_items(Iterator begin, Iterator end);
private:
    explicit AcceptableDnskeysReply(Api::Supervisor::AcceptableDnskeysReply& data);
    Api::Supervisor::AcceptableDnskeysReply& data_;
    friend AcceptableDnskeysReply make_wrapper(Api::Supervisor::AcceptableDnskeysReply& data);
};
AcceptableDnskeysReply make_wrapper(Api::Supervisor::AcceptableDnskeysReply& data);

class RawScanResultsReply : public RawScanResultsReplyWrapper<RawScanResultsReply>
{
public:
    using Iterator = Report::RawScanResultsReply::const_iterator;
    void set_items(Iterator begin, Iterator end);
private:
    explicit RawScanResultsReply(Api::Report::RawScanResultsReply& data);
    Api::Report::RawScanResultsReply& data_;
    friend RawScanResultsReply make_wrapper(Api::Report::RawScanResultsReply& data);
};
RawScanResultsReply make_wrapper(Api::Report::RawScanResultsReply& data);

}//namespace CdnskeyProcessor::Wrapper
}//namespace CdnskeyProcessor

#endif//WRAPPER_HH_BFA7226DF6F9ED86863B19DCC4F1DAE8
