/*
 * Copyright (C) 2021-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __CONTEXT__C3F7F7A016D23F370B9192E9D1464594
#define __CONTEXT__C3F7F7A016D23F370B9192E9D1464594

#include "import_results_from_file/log/context.hh"

#include <iostream>

namespace CdnskeyProcessor {
namespace Test {
namespace Log {

namespace {

std::string make_context_str(const std::initializer_list<std::string> _context_hierarchy)
{
    std::string result = get_context();
    static const std::string delimiter = " ";
    bool needs_delimeter = true;
    if (result.empty())
    {
        needs_delimeter = false;
    }
    for (const auto& context_element : _context_hierarchy)
    {
        if (needs_delimeter)
        {
            result += delimiter + context_element;
        }
        else
        {
            result += context_element;
            needs_delimeter = true;
        }
    }
    return result;
}

std::string& context()
{
    thread_local std::string context;
    return context;
}

void set_context(std::string _context)
{
    context() = std::move(_context);
}

} // namespace LibLog::{anonymous}

SetContext::SetContext(const std::initializer_list<std::string> _context_hierarchy)
    : previous_context_{get_context()}
{
    set_context(make_context_str(_context_hierarchy));
}

void SetContext::add(const std::initializer_list<std::string> _context_hierarchy)
{
    set_context(make_context_str(_context_hierarchy));
}

SetContext::~SetContext()
{
    set_context(std::move(previous_context_));
}

const std::string& get_context()
{
    return context();
}

} // namespace CdnskeyProcessor::Test::Log
} // namespace CdnskeyProcessor::Test
} // namespace CdnskeyProcessor

#endif
