/*
 * Copyright (C) 2021-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __LOG__D61123DFE09AD40379C1AC62730BDC50
#define __LOG__D61123DFE09AD40379C1AC62730BDC50

#include <liblog/liblog.hh>
#include "import_results_from_file/log/context.hh"

#define CDNSKEY_PROCESSOR_TEST_LOG_CALL(SPDLOG, msg, ...) \
        SPDLOG("{} -- {} -- {}", \
                __func__, \
                CdnskeyProcessor::Test::Log::get_context(), \
                LibLog::format(msg, ## __VA_ARGS__))

#define CP_TEST_TRACE(msg, ...) CDNSKEY_PROCESSOR_TEST_LOG_CALL(SPDLOG_TRACE, msg, ## __VA_ARGS__)
#define CP_TEST_DEBUG(msg, ...) CDNSKEY_PROCESSOR_TEST_LOG_CALL(SPDLOG_DEBUG, msg, ## __VA_ARGS__)
#define CP_TEST_INFO(msg, ...) CDNSKEY_PROCESSOR_TEST_LOG_CALL(SPDLOG_INFO, msg, ## __VA_ARGS__)
#define CP_TEST_WARNING(msg, ...) CDNSKEY_PROCESSOR_TEST_LOG_CALL(SPDLOG_WARN, msg, ## __VA_ARGS__)
#define CP_TEST_ERROR(msg, ...) CDNSKEY_PROCESSOR_TEST_LOG_CALL(SPDLOG_ERROR, msg, ## __VA_ARGS__)
#define CP_TEST_CRITICAL(msg, ...) CDNSKEY_PROCESSOR_TEST_LOG_CALL(SPDLOG_CRITICAL, msg, ## __VA_ARGS__)

#define CP_TEST_SET_CONTEXT(cls, var, ...) \
struct cls \
{ \
    CdnskeyProcessor::Test::Log::SetContext ctx; \
    cls(std::initializer_list<std::string> _context_hierarchy) : ctx{_context_hierarchy} \
    { \
        CP_TEST_TRACE("context entered"); \
    } \
    ~cls() \
    { \
        try \
        { \
            LIBLOG_TRACE("leaving context"); \
        } \
        catch (...) { } \
    } \
} var{__VA_ARGS__}


#endif
