/*
 * Copyright (C) 2021-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __SCAN_QUEUE__F2FC149C195BA00221527582352672BB
#define __SCAN_QUEUE__F2FC149C195BA00221527582352672BB

#include <src/api/domain_scan_task.hh>
#include <src/api/scan_batch_size.hh>
#include <src/domain.hh>
#include <src/internal/scan_result.hh>
#include <src/scan_queue_id.hh>
#include <src/timestamp.hh>

#include <map>

namespace CdnskeyProcessor {
namespace Test {

struct InvalidScanType : public std::exception
{
    const char* what() const noexcept override
    {
        return "invalid scan type";
    }
};

using ScanResultsPerWorker = std::map<WorkerName, ScanResultCollection>;
using ScanResultsPerDomain = std::map<Domain, ScanResultsPerWorker>;

using InsecureScanResults = std::map<Nameserver, ScanResultsPerDomain>;
using SecureScanResults = ScanResultsPerDomain;

class ScanQueue
{
public:
    ScanQueue();
    auto get_id() const -> ScanQueueId;
    void set_id(const ScanQueueId&);

    auto get_start_at() const -> Timestamp;
    auto get_finished_at() const -> Timestamp;
    auto get_scan_batch_size() const -> Interface::ScanBatchSize;

    auto get_scan_tasks() const -> const DomainScanTaskCollection&;
    void add_scan_task(const DomainScanTask&);

    auto get_insecure_scan_results() -> InsecureScanResults&;
    auto get_secure_scan_results() -> SecureScanResults&;
    void add_scan_result(const Domain&, const WorkerName&, const ScanResult&);

    void log() const;

private:
    ScanQueueId id;
    Interface::ScanBatchSize batch_size;
    Timestamp start_at;
    Timestamp finished_at;

    DomainScanTaskCollection tasks;
    InsecureScanResults insecure_results;
    SecureScanResults secure_results;
};

using ScanQueues = std::map<ScanQueueId, ScanQueue>;

void log_scan_queues(const ScanQueues&);
void log_scan_result(const ScanResult&);
void log_scan_result_collection(const ScanResultCollection&);

} // namespace CdnskeyProcessor::Test
} // namespace CdnskeyProcessor

#endif
